import pytest

from hgitaly.service.interceptors import (
    RequestLoggerInterceptor,
)
from hgitaly.service.blob import BlobServicer
from hgitaly.service.commit import CommitServicer
from hgitaly.service.ref import RefServicer
from hgitaly.service.diff import DiffServicer
from hgitaly.service.mercurial_changeset import MercurialChangesetServicer
from hgitaly.service.mercurial_repository import MercurialRepositoryServicer
from hgitaly.service.repository import RepositoryServicer
from hgitaly.service.server import ServerServicer
from hgitaly.stub.blob_pb2_grpc import add_BlobServiceServicer_to_server
from hgitaly.stub.commit_pb2_grpc import add_CommitServiceServicer_to_server
from hgitaly.stub.ref_pb2_grpc import add_RefServiceServicer_to_server
from hgitaly.stub.diff_pb2_grpc import add_DiffServiceServicer_to_server
from hgitaly.stub.repository_pb2_grpc import (
    add_RepositoryServiceServicer_to_server
)
from hgitaly.stub.mercurial_changeset_pb2_grpc import (
    add_MercurialChangesetServiceServicer_to_server
)
from hgitaly.stub.mercurial_repository_pb2_grpc import (
    add_MercurialRepositoryServiceServicer_to_server
)
from hgitaly.stub.server_pb2_grpc import add_ServerServiceServicer_to_server


@pytest.fixture(scope='module')
def grpc_interceptors():
    return [RequestLoggerInterceptor(),
            ]


@pytest.fixture(scope='module')
def grpc_server(_grpc_server, grpc_addr, server_repos_root):
    storages = dict(default=str(server_repos_root / 'default').encode())

    add_BlobServiceServicer_to_server(BlobServicer(storages),
                                      _grpc_server)
    add_CommitServiceServicer_to_server(CommitServicer(storages),
                                        _grpc_server)
    add_RefServiceServicer_to_server(RefServicer(storages),
                                     _grpc_server)
    add_DiffServiceServicer_to_server(DiffServicer(storages),
                                      _grpc_server)
    add_RepositoryServiceServicer_to_server(
        RepositoryServicer(storages),
        _grpc_server)
    add_MercurialChangesetServiceServicer_to_server(
        MercurialChangesetServicer(storages),
        _grpc_server)
    add_MercurialRepositoryServiceServicer_to_server(
        MercurialRepositoryServicer(storages),
        _grpc_server)
    add_ServerServiceServicer_to_server(ServerServicer(storages),
                                        _grpc_server)
    _grpc_server.add_insecure_port(grpc_addr)
    _grpc_server.start()
    yield _grpc_server
    _grpc_server.stop(grace=None)


@pytest.fixture(scope='module')
def server_repos_root(tmp_path_factory):
    return tmp_path_factory.mktemp("server-repos")
