use std::fs::File;
use std::io::Read;

use build_const::ConstWriter;

fn build_constants() -> Result<(), Box<dyn std::error::Error>> {
    // use `for_build` in `build.rs`
    let mut consts = ConstWriter::for_build("constants")?.finish_dependencies();
    let mut version = String::new();
    File::open("../../hgitaly/VERSION")?.read_to_string(&mut version)?;

    // Add a value that is a result of "complex" calculations
    consts.add_value("HGITALY_VERSION", "&str", version.trim());
    Ok(())
}

fn main() -> Result<(), Box<dyn std::error::Error>> {
    build_constants()?;
    tonic_build::configure()
        .build_server(true)
        .out_dir("src/generated")
        .protoc_arg("--experimental_allow_proto3_optional")
        .compile(
            &[
                "proto/commit.proto",
                "proto/repository.proto",
                "proto/server.proto",
            ],
            &[
                // include paths
                "proto",
                "../../protos",
                "../dependencies/rust-protobuf/proto",
            ],
        )
        .unwrap();
    Ok(())
}
