// Copyright 2023 Georges Racinet <georges.racinet@octobus.net>
//
// This software may be used and distributed according to the terms of the
// GNU General Public License version 2 or any later version.
// SPDX-License-Identifier: GPL-2.0-or-later
use std::env;
use std::path::PathBuf;

#[derive(Debug)]
pub struct Config {
    pub repositories_root: PathBuf,
    pub listen_url: String, // we don't plan supporting arbitrary bytes for a listen URL!
}

impl Config {
    /// Read configuration from environment variables
    ///
    /// This is useful for Gitaly Comparison tests, especially until we can
    /// support the HGitaly configuration file (i.e., HGRC)
    pub fn from_env() -> Self {
        Config {
            repositories_root: env::var_os("RHGITALY_REPOSITORIES_ROOT")
                .expect("RHGITALY_REPOSITORIES_ROOT not set in environment")
                .into(),
            listen_url: env::var("RHGITALY_LISTEN_URL")
                .expect("RHGITALY_LISTEN_URL not set in environment"),
        }
    }
}
